// Sniper_Scope 
// License: MIT 
// Author: Elusien (www.elusien.co.uk/shotcut)
 uniform vec2  Scope_Coords;   //= vec2(0.5, 0.5);            // Normalised coordinates of scope
 uniform float Scope_Diameter; //= 0.333;                     // Normalises diameter of scope (1.0 = video height)
 uniform vec4  Scope_Color;    //= vec4(0.18, 0.18, 0.3, 1,0);// Colour of the scope (normalised RGBA)
 uniform vec4  Reticule_Color; //= vec4(1.0 , 0.0 , 0.0, 1,0);// Colour of the scope (normalised RGBA)
 uniform float Mag_Index;      //= 1.0;						  // Degree of magnification (0.0 to 1.0)
 uniform float Scope_Thickness;//= 0.04;					  // Scope thickness fraction of scope diameter)
 uniform bool  Cross_Hairs;    //= true;					  // true = scope cross-hairs are shown
 uniform bool  Scope_Only;     //= false; 				      // true = only image within scope is shown

// Optimisation Routines - Courtesy of Daniel Holden (http://theorangeduck.com/page/avoiding-shader-conditionals)
float when_eq (float x, float y) {return 1.0 - abs(sign(x - y));}
float when_neq(float x, float y) {return       abs(sign(x - y));}
float when_gt (float x, float y) {return       max(sign(x - y), 0.0);}
float when_lt (float x, float y) {return       max(sign(y - x), 0.0);}
float when_ge (float x, float y) {return 1.0 - when_lt(x, y);}
float when_le (float x, float y) {return 1.0 - when_gt(x, y);}
vec2  when_eq (vec2  x, vec2  y) {return 1.0 - abs(sign(x - y));}
vec2  when_neq(vec2  x, vec2  y) {return       abs(sign(x - y));}
vec2  when_gt (vec2  x, vec2  y) {return       max(sign(x - y), 0.0);}
vec2  when_lt (vec2  x, vec2  y) {return       max(sign(y - x), 0.0);}
vec2  when_ge (vec2  x, vec2  y) {return 1.0 - when_lt(x, y);}
vec2  when_le (vec2  x, vec2  y) {return 1.0 - when_gt(x, y);}
vec3  when_eq (vec3  x, vec3  y) {return 1.0 - abs(sign(x - y));}
vec3  when_neq(vec3  x, vec3  y) {return       abs(sign(x - y));}
vec3  when_gt (vec3  x, vec3  y) {return       max(sign(x - y), 0.0);}
vec3  when_lt (vec3  x, vec3  y) {return       max(sign(y - x), 0.0);}
vec3  when_ge (vec3  x, vec3  y) {return 1.0 - when_lt(x, y);}
vec3  when_le (vec3  x, vec3  y) {return 1.0 - when_gt(x, y);}
vec4  when_eq (vec4  x, vec4  y) {return 1.0 - abs(sign(x - y));}
vec4  when_neq(vec4  x, vec4  y) {return       abs(sign(x - y));}
vec4  when_gt (vec4  x, vec4  y) {return       max(sign(x - y), 0.0);}
vec4  when_lt (vec4  x, vec4  y) {return       max(sign(y - x), 0.0);}
vec4  when_ge (vec4  x, vec4  y) {return 1.0 - when_lt(x, y);}
vec4  when_le (vec4  x, vec4  y) {return 1.0 - when_gt(x, y);}

float and     (float a, float b) {return     a * b      ;}
float or      (float a, float b) {return min(a + b, 1.0);}
float xor     (float a, float b) {return mod(a + b, 2.0);}
float not     (float a         ) {return     1.0 - a    ;}
vec2  and     (vec2  a, vec2  b) {return     a * b      ;}
vec2  or      (vec2  a, vec2  b) {return min(a + b, 1.0);}
vec2  xor     (vec2  a, vec2  b) {return mod(a + b, 2.0);}
vec2  not     (vec2  a         ) {return     1.0 - a    ;}
vec3  and     (vec3  a, vec3  b) {return     a * b      ;}
vec3  or      (vec3  a, vec3  b) {return min(a + b, 1.0);}
vec3  xor     (vec3  a, vec3  b) {return mod(a + b, 2.0);}
vec3  not     (vec3  a         ) {return     1.0 - a    ;}
vec4  and     (vec4  a, vec4  b) {return     a * b      ;}
vec4  or      (vec4  a, vec4  b) {return min(a + b, 1.0);}
vec4  xor     (vec4  a, vec4  b) {return mod(a + b, 2.0);}
vec4  not     (vec4  a         ) {return     1.0 - a    ;}

vec4 transition(vec2 p) { 
	vec2  SC          = vec2(Scope_Coords.x, (1.0 - Scope_Coords.y)); 
	float SD          = Scope_Diameter / ratio;			// Scope-Diameter as a fraction of frame height
	float SR          = SD * 0.5;						// Scope-Radius.
	float ST          = Scope_Diameter * Scope_Thickness;
	float R           = Scope_Diameter + (1.0 - Mag_Index) * (1.0 - Scope_Diameter);
	float CH          = float(Cross_Hairs);
	float CT          = CH * ST * 0.5;					// Cross-Hair thickness
	float RT          = CT * 0.25;						// Reticule thickness
	float NotSO       = not(float(Scope_Only));
	float hr          = R - R*sqrt(R - (SD*SD)/R);
	vec2  xy          = p - SC.xy;						// this point relative to scope centre.
	float rr          = sqrt(xy.x * xy.x * (ratio * ratio) + xy.y * xy.y);
	float rr_lt_SD    = when_lt(rr, SD);
	float ok2color    = or(NotSO, rr_lt_SD);			// 0 = Black, 1 = color
	
// Majority of the scene should be outside the scope (return color or Black).
	if (rr >= SD) { return vec4(vec3(ok2color), 1.0) * getFromColor(SC.xy + xy); }

// Below this line we are either on or inside the scope. Use the optimisation routines instad of IF THEN ELSE	
	float use_scope_colr =	or(
								when_gt(rr, SD-ST), CH * or(
									and(when_lt(abs(xy.x * ratio), CT), when_gt(abs(xy.y        ), SR)),
									and(when_lt(abs(xy.y        ), CT), when_gt(abs(xy.x * ratio), SR))
								)	
							);
	float use_retic_colr =	
							CH * or(
									and(when_lt(abs(xy.x * ratio), RT), when_le(abs(xy.y        ), SR)),
									and(when_lt(abs(xy.y        ), RT), when_le(abs(xy.x * ratio), SR))
	
							);
	
	float RR_gt_rr = when_gt(R , rr);
	float sqrt_arg = not(RR_gt_rr) * progress + RR_gt_rr * (R * R - rr * rr);	// prevent sqrt of -ve number error (<progress> is used simply to stop warning messages about it)
	vec4  colr     = getFromColor(SC.xy + xy * (R - hr) * inversesqrt(sqrt_arg));

	return use_scope_colr *    Scope_Color + 
	       use_retic_colr * Reticule_Color +
		   and(not(use_scope_colr), not(use_retic_colr)) * vec4(vec3(ok2color), 1.0) * colr;

}

